#ifndef __PACKETS_H__
#define __PACKETS_H__

const char FM_VERSION = 3;

const int FREEMEETING_PORT = 32570;

enum Tool { None, Vivavoz, VideoClient, VideoServer, Tgwb, MCastProxy,
            VVServer };

/* Base Control Packet */
class ControlPacket
{
    public:
        char version;
        char type;        
};

/* Base FreeMeeting Control Packet */
class FMControlPacket : public ControlPacket
{
    public:
        char tool;
        

        enum Type { FMConnectPacket = 21, FMDisconnectPacket = 22, 
                    FMShowHidePacket = 23, FMAckPacket = 24,
                    FMStatisticsPacket = 25, FMCallPacket = 26,
                    FMBusyPacket = 27, FMPortPacket = 28
                   };
};

/* Connect */
class FMConnectPacket : public FMControlPacket
{
    public:
        char destAddr[ 80 ];
};

/* Disconnect */
class FMDisconnectPacket : public FMControlPacket
{
    public:
        char username[ 80 ];
        char hostname[ 80 ];
};

/* Show/Hide window */
class FMShowHidePacket : public FMControlPacket
{
};

/* Ack Packet */
class FMAckPacket : public FMControlPacket
{
    public:
        char username[ 80 ];
        char hostname[ 80 ];
};

/* Timeout Packet */
class FMTimeoutPacket : public FMControlPacket
{
};

/* Statistics Packet */
class FMStatisticsPacket : public FMControlPacket
{
    public:
        unsigned long sent;
        unsigned long received;
};

/* Call Peer Packet */
class FMCallPacket : public FMControlPacket
{
    public:
        char username[ 80 ];
        char hostname[ 80 ];
};

/* Busy Packet */
class FMBusyPacket : public FMControlPacket
{
};

/* Port Packet: To warn FM which port tool is listening */
class FMPortPacket : public FMControlPacket
{
    public:
        int port;
};

#endif /* __PACKETS_H__ */
