/*
 * Copyright (C) 2005, Edmundo Albuquerque de Souza e Silva.
 *
 * This file may be distributed under the terms of the Q Public License
 * as defined by Trolltech AS of Norway and appearing in the file
 * LICENSE.QPL included in the packaging of this file.
 *
 * THIS FILE IS PROVIDED AS IS WITH NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#ifndef __BUFFER_H_
#define __BUFFER_H_

#include <pthread.h>

#define QTD_SLOTS   25

#define BUFFER_RECEIVED   0x01
#define BUFFER_RECOVERED  0x02
#define BUFFER_FLUSHED    0x04
#define BUFFER_REPLAYED   0x08
#define BUFFER_SILENCE    0x10

typedef struct SLOT_TAG
{
    char             flags;
    unsigned long    seqNumber;
    unsigned long    dataSize;
    unsigned char    data[ 1500 ];
    unsigned int     applLoss;
} SLOT, *PSLOT;

class Buffer
{
    private:
        pthread_mutex_t mutex;
        int             qtdItems;
        int             qtdSlots;
        SLOT            slot[ QTD_SLOTS ];
        bool            freeList[ QTD_SLOTS ];
        int             busyList[ QTD_SLOTS ];
        unsigned int    lastKnownDrops;

    public:
        Buffer( int );
        ~Buffer();
        int           insert( unsigned long, unsigned char *,
                              unsigned long, char );
        int           remove( unsigned long *, unsigned char *,
                              unsigned long *, char *, unsigned long *drops = NULL );
        void          getItem( unsigned long *, unsigned char *,
                               unsigned long *, char * );
        int           getQtdItems( void );
        unsigned long getFirstSeqNumber( void );
};

#endif
