/*
 * Copyright (C) 2005, Edmundo Albuquerque de Souza e Silva.
 *
 * This file may be distributed under the terms of the Q Public License
 * as defined by Trolltech AS of Norway and appearing in the file
 * LICENSE.QPL included in the packaging of this file.
 *
 * THIS FILE IS PROVIDED AS IS WITH NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
 
%{
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <arpa/inet.h>

#include "parser.h"

#define YYSTYPE char *
#define CONF_FILE "vvserver.conf"

//YYSTYPE yylval;

struct parameter curr_param;
extern char *yytext;
void yyerror( const char * );
int yylex( void );

int n_lines = 1;

%}

%token TK_IPADDR
%token TK_INTEGER
%token TK_EQ
%token TK_TRUE
%token TK_FALSE
%token TK_ID
%token TK_STRING
%token TK_DOT
%token TK_COLON
%token TK_DOUBLE_COLON

%%

configuration     : sections_list
                  ;

sections_list     : /* empty */
                  | section sections_list
                  ;

section           : variable TK_EQ value
                    {
                        int i;
                        
                        for( i=0; i<=n_items; i++)
                        {
                            if( strcmp( curr_param.variable, config[i].variable ) == 0 )
                            {
                                if( curr_param.type == config[i].type )
                                {
                                    switch( curr_param.type )
                                    {
                                        case TYPE_BOOLEAN:    
                                                config[i].v.bool_value = curr_param.v.bool_value;
                                                break;
                                                
                                        case TYPE_INTEGER:    
                                                config[i].v.int_value = curr_param.v.int_value; 
										        break;
                                                                                                                         
                                        /* TODO: OBS: Aqui no seria um memcpy??? */        
                                        case TYPE_IP_ADDRESS: 
                                                config[i].v.ip_value = curr_param.v.ip_value;
								                break;                                            
                                                
                                        case TYPE_STRING:                                                
                                        case TYPE_REDUN:
                                        case TYPE_ID:
                                                strcpy( config[i].v.str_value,
                                                        curr_param.v.str_value ); 
			                                       break;
                                    }
                                    config[i].used = 1;

                                    break; /* end of "for" loop */
                                }
                                else
                                {
                                    fprintf( stderr, "%s: %d: Type does not match.\n", CONF_FILE, n_lines );
                                    YYABORT;
                                }
                            }
                            else if( i == n_items )
                                 {
                                    fprintf( stderr, "%s: %d: Identifier %s does not exist.\n", 
                                             CONF_FILE, n_lines, curr_param.variable ); 
                                    YYABORT;
                                 }
                        }
                    }                    
                  ;

variable          : TK_ID
                    {
                        $$ = strdup( yytext ); 
                        strcpy( curr_param.variable, yytext );
                    }
                  ;
              
value             : boolean
                    {
                        curr_param.type = TYPE_BOOLEAN;
                    }
                    
                  | ip_address
                    {
                         curr_param.type = TYPE_IP_ADDRESS; 
                    }
                    
                  | schema_list
                    {
                        curr_param.type = TYPE_REDUN;
                    }
                        
                  | TK_INTEGER
                    {
                        curr_param.type = TYPE_INTEGER;
                        $$ = strdup( yytext );                        
                        curr_param.v.int_value =  atoi( yytext ); 
                    }                    
                                      
                  | TK_STRING
                    {
                        curr_param.type = TYPE_STRING;
                        /*curr_param.type = TYPE_STRING;*/
                        $$ = strdup( yytext );
                        curr_param.v.str_value = strdup( yytext );

                        /* Removing the quotes from the string */
                        curr_param.v.str_value[ (strlen( curr_param.v.str_value ) - 1 ) ] = '\0';
                        strcpy( curr_param.v.str_value, &curr_param.v.str_value[1] );
                    }  
                   
                   | TK_ID
                    {
                        $$ = strdup( yytext );
                        curr_param.v.str_value = strdup( yytext );

                        if( strcmp( curr_param.v.str_value, "NONE" ) == 0 )
                            curr_param.type = TYPE_REDUN;
                        else
                            curr_param.type = TYPE_ID;
                    }                  
    
                  ;
                  
schema_list       : schema_list TK_DOUBLE_COLON schema
                    {
                        strcpy( curr_param.v.str_value, $1 ); 
                        strcat( curr_param.v.str_value, "::" );
                        
                        strcat( curr_param.v.str_value, $3 );                        
                    }
                  
                  | schema
                                      
                  ;
                  
schema            : TK_INTEGER TK_COLON TK_INTEGER
                    {
                        strcpy( curr_param.v.str_value, $1 ); 
                        strcat( curr_param.v.str_value, ":" );
                        
                        strcat( curr_param.v.str_value, $3 );
                        
                        $$ = strdup( curr_param.v.str_value );                        
                    }                        
                  
                  |  
                  
                  ;

boolean           : TK_TRUE
                    {
                        curr_param.v.bool_value = '1';
                    }
                    
                  | TK_FALSE
                    {
                         curr_param.v.bool_value = '0';
                    }
                  ;

ip_address        : TK_IPADDR 
                    {
                        $$ = strdup( yytext );                       
                        curr_param.v.ip_value.sin_addr.s_addr = inet_addr( yytext );                         
                    }
                  ;
                   
%%
void yyerror( const char *msg )
{
    fprintf( stderr, "Error: %s\n", msg );
    exit( -1 );
}
