/*
 * Copyright (C) 2005, Edmundo Albuquerque de Souza e Silva.
 *
 * This file may be distributed under the terms of the Q Public License
 * as defined by Trolltech AS of Norway and appearing in the file
 * LICENSE.QPL included in the packaging of this file.
 *
 * THIS FILE IS PROVIDED AS IS WITH NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "vvCodecs.h"
#include "globals.h"
#include "forward.h"
#include "net.h"
#include "trace.h"
#include "buffer.h"
#include "stats.h"
#include "mix.h"

extern "C"
{
#include "redun.h"
};

extern USERINFO  vvserverInfo;
extern char      generateTrace;

int WAITBUFFER;
unsigned long sendSeqNumber = 0;   /* sequence number */

/**
 * Sends the packet pct
 */
static void sendPckt( int soc, char *pct, int qtdbytes, PDESTINATION cur )
{
    int bysent;

    #ifdef _DEBUG
    fprintf( stderr, "sendPckt(): sending mixed packet\n" );
    #endif

    bysent = sendto( soc, pct, qtdbytes, 0, (struct sockaddr *)&cur->addr,
            sizeof( cur->addr ) );

    if( bysent > 0 )
    {
        cur->packets.sent++;
        vvserverInfo.packets.sent++;
    }
}

/**
 * Thread that waits for audio packets.
 * If necessary recvaudio also recovers lost packets
 */
void recvAudio( PCONNECTINFO connInfo )
{
    short byread;
    PDATAHEADER dHeader;
    int length;
    unsigned short offset;
    unsigned short recvsize;
    unsigned long int recvSeqNumber;
    static unsigned int pcksReceived;
    struct sockaddr_in from;
    byte src[ MAXPCKTSIZE ];
    BUFFSLOT buff;
    vvCodecs codecs( MODE_DECODER );

    PDESTINATION cur;
    
    /*Debug*/
    int status;

    traceStartTimer();

    /* initialize playout buffer structures */
    WAITBUFFER = TRUE;

    pcksReceived = 0;
    length = sizeof( from );

    while( 1 )
    {
        if( (byread = recvfrom ( connInfo->socData, src, MAXPCKTSIZE, 0,
                                (struct sockaddr *)&from,
                                (socklen_t *)&length )) > 0 )
        {
            /* Is this packet from someone in the group? */
            cur = connInfo->dests;
            while( cur != NULL )
            {
                if( cmpIP( (char *)&cur->addr, (char *)&from ) == 0 )
                    break;

                cur = cur->next;
            }

            if( cur != NULL )
            {
                /* Increments total of packets received */
                vvserverInfo.packets.received++;

                dHeader = (PDATAHEADER)src;
                if( (dHeader->type & TYPE_MASK) == DATA_PACKET )
                {
                    recvSeqNumber = ntohl( dHeader->seqNumber );
                    
                    if( generateTrace )
                        traceAddLine( cur, recvSeqNumber, byread );

                    /* Remove jitter */
                    buff.codec    = dHeader->codec;
                    buff.seqNumber = recvSeqNumber;
                    buff.dataSize  = vvCodecs::getSize( dHeader->codec );
                    memcpy( buff.audio, (char *)&src[ sizeof( DATAHEADER ) ],
                            buff.dataSize );
                    status = cur->buffer->insert( buff.seqNumber,
                                                  (unsigned char *)&buff,
                                                  sizeof( BUFFSLOT ), BUFFER_RECEIVED );

                    offset = buff.dataSize + sizeof( DATAHEADER );

                    redun_putData( cur->recvRedun, (char *)buff.audio,
                                   buff.dataSize, recvSeqNumber );

                    /* Check if we have redundancies */
                    if( dHeader->type & WITH_FEC )
                    {
                        redun_putFECPacket( cur->recvRedun, (char *)&src[offset],
                                            byread - offset );

                        recvsize = MAXPCKTSIZE;
                        while( redun_getRecovered( cur->recvRedun,
                                                   (char *)buff.audio,
                                                   &recvsize,
                                                   &buff.seqNumber ) )
                        {
                            buff.dataSize = recvsize;
                            buff.codec    = vvCodecs::getFromSize( buff.dataSize );

                            status = cur->buffer->insert( buff.seqNumber,
                                                          (unsigned char *)&buff,
                                                          sizeof( BUFFSLOT ),
                                                          BUFFER_RECOVERED );

                            recvsize = MAXPCKTSIZE;
                        }
                    }
                }

                if( dHeader->flags & SILENCE )
                    cur->isSilent = TRUE;

            } /* if( cur!=null )*/
            length = sizeof( from );

        } /* if -> recvfrom */
    } /* while(1) */
} /* recvAudio() */

/**
 * Builds the packet that id to be sent.
 * @param dest - contains the packet data
 *        audio - contains the packets audio
 *        cur - contains the address of the client to whom the packet will be
 *              sent
 *        seqNumber - packet sequence number
 */
int buildPckt( byte *dest, unsigned char *audio, PDESTINATION cur, 
               unsigned long * seqNumber )
{
    char *ptr;                    /* ptr for help build the pck to be sent    */
    int pcktSize;                 /* final packet size to be delivered        */
    int dataSize;
    DATAPCKT dataPckt;
    byte fec[ MAXPCKTSIZE ];       /* Buffer that will have the FEC packet    */
    unsigned short fecsize;        /* size of the FEC part including header   */
//    unsigned long sendSeqNumber;   /* sequence number */
    REDUNSCHEME scheme[ 2 ];
    PDATAHEADER dHeader;

//    sendSeqNumber = 0;

    /* Initialize redundancy scheme */
    if( cur->sendRedun == NULL )
    {
        scheme[0].windowsize = 2;
        scheme[0].no_chains  = 1;
        scheme[1].windowsize = 6;
        scheme[1].no_chains  = 3;
        cur->sendRedun = redun_CreateSendChain( &scheme[0], vvserverInfo.redun,
                                                sizeof( DATAPCKT ) );
    }

    dataPckt.header.type  = DATA_PACKET;
    dataPckt.header.codec = vvserverInfo.codec;
    dataPckt.header.flags = 0;

    pcktSize = sizeof( DATAHEADER );
    dataSize = cur->codecs->encode( (char *)audio, vvserverInfo.codec,
                                    (char *)dataPckt.audiosample );

    pcktSize += dataSize;
    memcpy( dest, (unsigned char *)&dataPckt, pcktSize );

    cur->sendRedun->flags = vvserverInfo.redun ?
                            (cur->sendRedun->flags | RF_ON) :
                            cur->sendRedun->flags & ~RF_ON;

    /* Now it's time to add redundancies */
    dHeader = (PDATAHEADER)dest;

    fecsize = MAXPCKTSIZE;
    if( redun_buildFEC( cur->sendRedun, (char *)dataPckt.audiosample, dataSize,
                        (char *)fec, &fecsize, &sendSeqNumber ) )
    {
        dHeader->type |= WITH_FEC;
        ptr = (char *)dest;
        ptr += pcktSize;
        memcpy( ptr, fec, fecsize );
        pcktSize += fecsize;
    }
    dHeader->seqNumber = htonl( sendSeqNumber );

    return( pcktSize );
}

/**
 * Builds the audio to be sent to each client.
 * This function is divided into two parts:
 * 1) For each client connected to vvserver, sendAudio mixes the audio received
 *    from all other partys connected and stores it in the mixedAudio variable
 *    of the clients correspondig DESTINATION_TAG structure.
 * 2) Calls the sendPckt function for each connected client
 */
static void sendAudio( PCONNECTINFO connInfo )
{
    int pckt_size, status;
    unsigned long drops;
    byte dest[ MAXPCKTSIZE ];
    BUFFSLOT zombie, pckt;
    PBUFFSLOT buff;
    PDESTINATION cur, aux;
    
    buff = (PBUFFSLOT)malloc( sizeof( BUFFSLOT ) );

    /* building the audio to be sent */
    cur = connInfo->dests;
    while( cur != NULL )
    {
        if( cur->state == ESTABLISHED )
        {
            /* check if the client's buffer has audio */
            if( cur->buffer->getQtdItems() != 0 )
            {
                aux = connInfo->dests;

                while( aux != NULL )
                {
                    if( cmpIP( (char *)&cur->addr, (char *)&aux->addr ) != 0 )
                    {
                        if( aux->next != NULL )
                        {
                            /* getting new audio */
                            cur->buffer->getItem( &pckt.seqNumber,
                                                  (unsigned char *)buff,
                                                  &pckt.dataSize, &pckt.flags );
                        }
                        /* We can remove audio from the buffers */
                        else
                        {
                            status = cur->buffer->remove( &pckt.seqNumber,
                                                          (unsigned char *)buff,
                                                          &pckt.dataSize, &pckt.flags, &drops );
                                                 
                            if( pckt.seqNumber >= cur->ctrlStat.lastSeqNumberPlayed )
                            {
                                /* Now I should calculated the stats */
                                calculateStats( cur, pckt.seqNumber, pckt.flags, drops );
                            }
                        }

                        cur->codecr->decode( (char *)buff->audio, buff->codec,
                                            (char *)pckt.audio );

                        if( aux->numbOfMixes == 0 )
                            memcpy( aux->mixedAudio,
                                    (unsigned char *)pckt.audio,
                                    sizeof( buff->audio ) );
                        else
                            mix( aux->mixedAudio, pckt.audio,
                                 aux->numbOfMixes + 1 );

                        aux->numbOfMixes++;
                    }
                    else
                    {
                        /* done only to remove the audio from the */
                        /* buffer of the last client of the list. */
                        if( cur->next == NULL )
                        {
                            status = cur->buffer->remove( &zombie.seqNumber,
                                                          (unsigned char *)&zombie.audio,
                                                          &zombie.dataSize, &zombie.flags, &drops );
                            
                            if( zombie.seqNumber >= cur->ctrlStat.lastSeqNumberPlayed )
                            {
                                /* Now I should calculated the stats */
                                calculateStats( cur, zombie.seqNumber, zombie.flags, drops );
                            }
                        }
                    }
                    aux = aux->next;
                }
            }
        }
        cur = cur->next;
    }

    /* sending audio */
    cur = connInfo->dests;
    while( cur != NULL )
    {
        if( cur->numbOfMixes != 0 )
        {
            pckt_size = buildPckt( dest, cur->mixedAudio, cur, 
                                   &sendSeqNumber );
    
            sendPckt( connInfo->socData, (char *)dest, pckt_size, cur );
        }

        /* cleaning variables */
        bzero( cur->mixedAudio, PCMDATASIZE );
        cur->numbOfMixes = 0;

        cur = cur->next;
    }
}

/**
 * Checks if someone is speaking
 */
void forwardData( PCONNECTINFO connInfo )
{
    int buffer_flag, silence_flag;
    PDESTINATION cur;

    buffer_flag  = FALSE;
    silence_flag = TRUE;
    cur = connInfo->dests;
    while( cur != NULL )
    {
        if( cur->buffer->getQtdItems() >= NMIN )
        {
             buffer_flag = TRUE;
             break;
        }
        else
            if( !cur->isSilent )
                silence_flag = FALSE;

        cur = cur->next;
    }

    /* sending pending packets if everyone is silent */
//    if( cur == NULL && !silence_flag )
//        flushBuffer( connInfo );

    if( buffer_flag )
        sendAudio( connInfo );
}
