/*
 * Copyright (C) 2005, Edmundo Albuquerque de Souza e Silva.
 *
 * This file may be distributed under the terms of the Q Public License
 * as defined by Trolltech AS of Norway and appearing in the file
 * LICENSE.QPL included in the packaging of this file.
 *
 * THIS FILE IS PROVIDED AS IS WITH NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
 
#ifndef __GLOBALS_H__
#define __GLOBALS_H__

#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <time.h>

extern "C"
{
#include "redun.h"
};

#include "buffer.h"

/* End of Transmission */
#define EOT     1

/* Default port for control packets. Audio port is obtained by adding 1 */
#define DEFAULT_PORT 32564

/* Number of pcts before start playing */
#define NMIN            8

/* Connection type */
#define TYPE_DISCONNECTED   0
#define TYPE_MULTICAST      1
#define TYPE_UNICAST        2

#ifndef TRUE
  #define TRUE   1
  #define FALSE  0
#endif


#ifndef __BYTE__
#define __BYTE__
 typedef unsigned char byte;
#endif

typedef union FLOATLONG_TAG
{
    float tfloat;
    long  tlong;
} *PFLOATLONG, FLOATLONG;

typedef struct BUFFSLOT_TAG
{
    char          flags;
    unsigned long seqNumber;
    unsigned char codec;
    double        arrivalTime;
    unsigned long dataSize;
    unsigned char audio[ PCMDATASIZE ];
} *PBUFFSLOT, BUFFSLOT;

typedef struct STATS_TAG
{
    char           flags;
    unsigned long  sent;         /** number of sent packets                   */
    unsigned long  lost;         /** the number of lost packets               */
    unsigned long  received;     /** the number of received packets           */
    unsigned long  recovered;    /** the number of packets recoverd by the    */
                                 /** error correction algorithm               */
    unsigned long  dropped;      /* the number of packets dropeed by the      */
                                 /* application due to audio queue congestion */
    unsigned long  outoforder;   /** Number of pcts that arrived out of order */
    unsigned long  notplayed;    /** Number of received but not played pcts   */
    FLOATLONG      successMean;
    FLOATLONG      lostMean;
} *PSTATS, STATS;

typedef struct CTRLSTAT_TAG
{
    unsigned int   successSize;
    unsigned int   successNumber;
    float          successMean;
    unsigned int   lostSize;
    unsigned int   lostNumber;
    float          lostMean;
    unsigned long  lastSeqNumberRecv;
    unsigned long  lastSeqNumberPlayed;
} *PCTRLSTAT, CTRLSTAT;

typedef struct USERINFO_TAG
{
    char  username[ 80 ];       /** user name string                   */
    char  hostname[ 80 ];       /** machine name string                */
    unsigned char  codec;       /** the codec being used by the user   */
    int   format;               /** format returned by audio device    */
    STATS packets;              /** packets statistics                 */
    char  redun;
    char  silenceUser;
} *PUSERINFO, USERINFO;


typedef struct TRACEINFO_TAG
{
    char filename[ 80 ];
    FILE *out;
} *PTRACEINFO, TRACEINFO;

typedef struct DESTINATION_TAG
{
    unsigned char state;         /** connection state                       */
    STATS     packets;      
    CTRLSTAT  ctrlStat;
    struct sockaddr_in addr;     /** destination address                    */
    PCHAIN    recvRedun;         /** redundancy chain for receiving audio   */
    PCHAIN    sendRedun;         /** redundancy chain for sending audio     */
    time_t    lastStatusPckt;    /** time of last status packet             */
    Buffer   *buffer;            /** stores the audio send by addr          */
    int       isSilent;       
    TRACEINFO traceInfo;         /** stores trace information               */
    unsigned char mixedAudio[ PCMDATASIZE ];   
                                 /** stores the audio to be sent to addr    */
    vvCodecs *codecr;
    vvCodecs *codecs;
    int numbOfMixes;             /** number of audios mixed in mixedAudio   */                   

    char  hostname[ 80 ];
    char  username[ 80 ];

    struct DESTINATION_TAG *next;
} *PDESTINATION, DESTINATION;

typedef struct CONNECTINFO_TAG
{
    int            audioFormat; /** sample format returned by audio    */
    int            socCtrl;     /** file descriptor for control socket */
    int            socData;     /** file descriptor for audio socket   */
    int            type;        /** connection type                    */
    unsigned char  connected;   /** connection state                   */
    unsigned int   port;        /** connection port                    */
    unsigned char  TimeToLive;  /** time to live for multicast packets */
    PDESTINATION   dests;       /** list of the connected peers        */
    char logDir[200];           /** directory where log file will be 
                                    created                            */
    struct sockaddr_in FMdest;  /** FreeMeeting                        */
} *PCONNECTINFO, CONNECTINFO;

#ifdef SUNOS
    int gethostname( char *, int );
#endif

int cmpIP( char *, char * );
void printIP( char * );

#ifdef _DEBUG
void printConnectionList( PCONNECTINFO );
#endif

#endif
