/*
 * Copyright (C) 2005, Edmundo Albuquerque de Souza e Silva.
 *
 * This file may be distributed under the terms of the Q Public License
 * as defined by Trolltech AS of Norway and appearing in the file
 * LICENSE.QPL included in the packaging of this file.
 *
 * THIS FILE IS PROVIDED AS IS WITH NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#ifndef __REDUN_H__
#define __REDUN_H__

#define REDUNSIGNATURE  0xA3
#define RECOVEREDPACKET 0xFE
#define REQUESTEDPACKET 0XFF

/* Flags */
#define RF_SEND           1
#define RF_RECV           2
#define RF_ON          0x80

typedef struct
{
    char busy;
    unsigned short size;
} RDATAHEADER, *PRDATAHEADER;

typedef struct
{
    unsigned char no_chains;
    unsigned char windowsize;
} REDUNSCHEME, *PREDUNSCHEME;

typedef struct
{
    unsigned char signature;
    unsigned char no_reduns;
    unsigned long SN;
    unsigned short pctsize;
} FECHEADER, *PFECHEADER;

typedef struct
{
    unsigned long  base;
    unsigned int   mask;
    unsigned short size;
} REDUNHEADER, *PREDUNHEADER;

typedef struct
{
    int windowsize;
    int interleave;
    int index;
    char isFirst;
    char numPcts;        /* number of pcts already inserted in this chain */
    char completed;      /* determine whether xor calculation is completed*/
    char toBeDeleted;    /* This chain will be deleted in the next window */
    char zombie;         /* 1 if this chain can be deleted, 0 otherwise.  */
    unsigned long  base;
    unsigned int   mask;
    unsigned short size; /* size of the greatest packet in this chain     */
    unsigned char *xor_data;
} SENDCHAIN, *PSENDCHAIN;

typedef struct
{
    long firstFECSN;
    long firstDataSN;
    long fecbuffersize;
    long databuffersize;
    unsigned short no_slots;
    long dataslotsize;
    long fecslotsize;
    unsigned char *xor_data;
    unsigned char **fecslot;
    unsigned char *fecbuffer;
    unsigned char **dataslot;
    unsigned char *databuffer;
    PRDATAHEADER dataheader;
} RECVCHAIN, *PRECVCHAIN;

typedef struct
{
    unsigned int flags;
    long Data_SN;
    long FEC_SN;
    long pctsize;
    int  no_sChains;
    PSENDCHAIN sChain;
    RECVCHAIN  rChain;
} CHAIN, *PCHAIN;

/* Prototypes */
void   redun_ChangeSendChain( PCHAIN, PREDUNSCHEME, unsigned int, unsigned int);
PCHAIN redun_Create( PREDUNSCHEME, unsigned int, unsigned int, unsigned int );
PCHAIN redun_CreateSendChain( PREDUNSCHEME, unsigned int, unsigned int );
PCHAIN redun_CreateRecvChain( unsigned int, unsigned int );
void   redun_Destroy( PCHAIN );
void   redun_DelayedChangeChain( PCHAIN, PREDUNSCHEME, int );
int    redun_putData( PCHAIN, char *, unsigned short, unsigned long );
void   redun_putFECPacket( PCHAIN, char *, unsigned short );
int    redun_getRecovered( PCHAIN, char *, unsigned short *, unsigned long * );
int redun_buildFEC( PCHAIN, char *, unsigned short, char *, unsigned short *, unsigned long * );

#endif
