/*
 * Copyright (C) 2005, Edmundo Albuquerque de Souza e Silva.
 *
 * This file may be distributed under the terms of the Q Public License
 * as defined by Trolltech AS of Norway and appearing in the file
 * LICENSE.QPL included in the packaging of this file.
 *
 * THIS FILE IS PROVIDED AS IS WITH NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
 
#include "vvCodecs.h"
#include "globals.h"
#include "stats.h"

void initStats( PDESTINATION dst )
{
    if( dst != NULL )
    {
        dst->ctrlStat.successSize   = 0;
        dst->ctrlStat.successNumber = 0;
        dst->ctrlStat.successMean   = 0.0;
        dst->ctrlStat.lostSize      = 0;
        dst->ctrlStat.lostNumber    = 0;
        dst->ctrlStat.lostMean      = 0.0;
        dst->ctrlStat.lastSeqNumberRecv   = 0;
        dst->ctrlStat.lastSeqNumberPlayed = 0;
    }
}

unsigned long getLastSeqNumber( PDESTINATION dst )
{
    return( dst->ctrlStat.lastSeqNumberPlayed );
}

void calculateStats( PDESTINATION dst, unsigned long recvNumber, char flags, unsigned long drops )
{
    if( !(flags & BUFFER_REPLAYED) )
    {
        if( flags & BUFFER_RECEIVED )
        {
            dst->packets.received++;
            dst->packets.dropped += drops;

            if( recvNumber > dst->ctrlStat.lastSeqNumberRecv )
            {
                dst->packets.lost += ( recvNumber - 
                                     dst->ctrlStat.lastSeqNumberRecv - drops - 1 );

//                 fprintf( stderr, "Acabei de perder %u pacotes: recvNum = %u, lastRecv = %u, lost = %u\n",
//                          ( recvNumber - dst->ctrlStat.lastSeqNumberRecv - 1 ),
//                          recvNumber, dst->ctrlStat.lastSeqNumberRecv,
//                          dst->packets.lost );

                if( recvNumber - dst->ctrlStat.lastSeqNumberRecv == 1 )
                {
                    dst->ctrlStat.successSize++;
                }
                else
                {
                    dst->ctrlStat.lostSize = recvNumber - 
                                             dst->ctrlStat.lastSeqNumberRecv -
                                             drops - 1;
                    dst->ctrlStat.lostNumber++;
                    dst->ctrlStat.lostMean = (dst->ctrlStat.lostMean * 
                                             (dst->ctrlStat.lostNumber-1) +
                                              dst->ctrlStat.lostSize) / 
                                              dst->ctrlStat.lostNumber;
                    dst->ctrlStat.successNumber++;
                    dst->ctrlStat.successMean = (dst->ctrlStat.successMean *
                                             (dst->ctrlStat.successNumber - 1) +
                                             dst->ctrlStat.successSize) / 
                                             dst->ctrlStat.successNumber;
                    dst->ctrlStat.successSize = 0;
                    dst->packets.successMean.tfloat = dst->ctrlStat.successMean;
                    dst->packets.lostMean.tfloat    = dst->ctrlStat.lostMean;
                }
                
                dst->ctrlStat.lastSeqNumberRecv = recvNumber;
            }
            else
            {
                dst->packets.outoforder++;
            }
        }
        else /* BUFFER_RECEIVED */
        {
            if( flags & BUFFER_RECOVERED )
                dst->packets.recovered++;
        }
        dst->ctrlStat.lastSeqNumberPlayed = recvNumber;
    }
}
