/*
 * Copyright (C) 2005, Edmundo Albuquerque de Souza e Silva.
 *
 * This file may be distributed under the terms of the Q Public License
 * as defined by Trolltech AS of Norway and appearing in the file
 * LICENSE.QPL included in the packaging of this file.
 *
 * THIS FILE IS PROVIDED AS IS WITH NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
 
#include <sys/time.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include "vvCodecs.h"
#include "globals.h"
#include "trace.h"

double start_time;

inline double get_time()
{
    struct timeval time;

    gettimeofday( &time, NULL );
    return (double)time.tv_sec * 1000000 + time.tv_usec;
} 

void initializeTraceInfo( PDESTINATION dest, char * hostname )
{
    time_t curtime;
    struct tm *loctime;
    char date[80];

    /* Get the current time. */
    curtime = time (NULL);

    /* Convert it to local time representation. */
    loctime = localtime (&curtime);
    strftime( date, 80, "%d-%m-%Y.%H:%M:%S", loctime );

    dest->traceInfo.out = NULL;
    strcpy( dest->traceInfo.filename, "vvserver-" );

    strcat( dest->traceInfo.filename, hostname );
    
    strcat( dest->traceInfo.filename, "." );
    strcat( dest->traceInfo.filename, date );
    strcat( dest->traceInfo.filename, ".trace" );
}

void traceStartTimer()
{
    start_time = get_time();

#ifdef _SUGATA_
    fprintf( stderr, "Begin = %7.0f\n", start_time );
#endif
}

void traceAddLine( PDESTINATION dest, unsigned long int seqNumber, short size )
{
    if( dest->traceInfo.out )
    {
        fprintf( dest->traceInfo.out, "\t%ld \t\t\t", seqNumber );
        fprintf( dest->traceInfo.out, "%7.0f \t\t\t",
                 get_time() - start_time );
        fprintf( dest->traceInfo.out, "%d \n", size );
        fflush( dest->traceInfo.out );
    }
}

void traceOpen( PDESTINATION dest, char* dir, char * hostname )
{
    char filename[80];
    
    if( dest->traceInfo.out == NULL )
    {
        initializeTraceInfo( dest, hostname );
                            
        strcpy( filename, dest->traceInfo.filename );                             
        strcpy( dest->traceInfo.filename, dir );
        strcat( dest->traceInfo.filename, "/" );
        strcat( dest->traceInfo.filename, filename );  
        
        if( ( dest->traceInfo.out = fopen( dest->traceInfo.filename, "w" )) != NULL )
        {
            fseek( dest->traceInfo.out, 0, SEEK_END );
            if( ftell( dest->traceInfo.out ) == 0 )
            {
                fprintf( dest->traceInfo.out, "Sequence Number \t");
                fprintf( dest->traceInfo.out, "Arrived Time of Packets (us) \t");
                fprintf( dest->traceInfo.out, "Size of Packets (bytes)\n");
                fflush( dest->traceInfo.out );
            }
        }        
    }    
}

void traceClose( PDESTINATION dest )
{
    if( dest->traceInfo.out )
    {
        fclose( dest->traceInfo.out );
        dest->traceInfo.out = NULL;
    }
}
