/*
 * Copyright (C) 2005, Edmundo Albuquerque de Souza e Silva.
 *
 * This file may be distributed under the terms of the Q Public License
 * as defined by Trolltech AS of Norway and appearing in the file
 * LICENSE.QPL included in the packaging of this file.
 *
 * THIS FILE IS PROVIDED AS IS WITH NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <string.h>

#include "vvCodecs.h"

int vvCodecs::speexSizes[] = { 6, 10, 15, 20, 20, 28, 28, 38, 38, 46, 62 };

vvCodecs::vvCodecs( bool mod )
{
    mode      = mod;
    lastCodec = 0;
    hgsm      = gsm_create();
    if( mod == MODE_DECODER )
        speex.state = speex_decoder_init( &speex_nb_mode );
    else
        speex.state = speex_encoder_init( &speex_nb_mode );
}

vvCodecs::~vvCodecs()
{
    gsm_destroy( hgsm );
    if( mode == MODE_DECODER )
        speex_decoder_destroy( speex.state );
    else
        speex_encoder_destroy( speex.state );
}

int vvCodecs::encode( char *pcmSample, unsigned char codec, char *dest )
{
    int i;
    int newSize;
    short *ptr;

    newSize = 0;
    switch( (codec & 0xF) )
    {
        case CODEC_GSM:
            gsm_encode( hgsm, (gsm_signal *)pcmSample, (gsm_byte *)dest);
            newSize = GSMDATASIZE;
            break;

        case CODEC_MULAW:
            ptr = (short *)pcmSample;
            for( i = 0; i < PCMDATASAMPLES; i++ )
//                dest[ i ] = (unsigned char)(linear16_2_ulaw[ ptr[ i ] + 32767]);
            newSize = MULAWDATASIZE;
            break;

        case CODEC_SPEEX:
            if( codec != lastCodec )
            {
                int quality = codec >> 4;
                speex_encoder_ctl( speex.state, SPEEX_SET_QUALITY, &quality );
                speex_bits_init( &speex.bits );
            }
            speex_bits_reset( &speex.bits );
            speex_encode_int( speex.state, (short *)pcmSample, &speex.bits );
            newSize = speex_bits_write( &speex.bits, dest, 200 );
            break;

        case CODEC_NONE:
            memcpy( dest, pcmSample, PCMDATASIZE );
            newSize = PCMDATASIZE;
            break;
    }

    lastCodec = codec;

    return( newSize );
}

int vvCodecs::decode( char *orig, unsigned char codec, char *dest )
{
    int offset;

    offset = 0;
    switch( (codec & 0xF) )
    {
        case CODEC_NONE:
            memcpy( dest, orig, PCMDATASIZE );
            offset = PCMDATASIZE;
            break;

        case CODEC_GSM:
            gsm_decode( hgsm, (gsm_byte *)orig, (short *)dest );
            offset = GSMDATASIZE;
            break;

        case CODEC_SPEEX:
            if( codec != lastCodec )
            {
                int tmp = codec >> 4;
                speex_decoder_ctl( speex.state, SPEEX_SET_ENH, &tmp );
                speex_bits_init( &speex.bits );
            }
            speex_bits_read_from( &speex.bits, orig, speexSizes[(codec >> 4)] );
            speex_decode_int( speex.state, &speex.bits, (short *)dest );
            offset = speexSizes[ (codec >> 4) ];
            break;
    }

    lastCodec = codec;

    return( offset );
}

int vvCodecs::getSize( unsigned char codec )
{
    int size;

    size = 0;
    switch( (codec & 0xF) )
    {
        case CODEC_NONE:  size = PCMDATASIZE;                break;
        case CODEC_GSM:   size = GSMDATASIZE;                break;
        case CODEC_SPEEX: size = speexSizes[ (codec >> 4) ]; break;
    }

    return( size );
}

int vvCodecs::getFromSize( int size )
{
    int codec, i;

    codec = 0;
    switch( size )
    {
        case PCMDATASIZE: codec = CODEC_NONE; break;
        case GSMDATASIZE: codec = CODEC_GSM;  break;
        default:
            for( i = 0; i < 11; i++ )
                if( size == speexSizes[ i ] )
                {
                    codec = (i << 4) | CODEC_SPEEX;
                    break;
                }
    }

    return( codec );
}
